/*
 * etiproto.h - header file defining extension of RFB protocol for the purposes 
 * of eye-tracking information processing.
 */

#pragma once

#include <map>


/*****************************************************************************
 *
 * Message types
 *
 *****************************************************************************/

/* server -> client */
//Gaze Data messages
#define etiServerGazeSample			201	//send new eye tracking sample
#define etiServerGazeFixation		202	//send new eye tracking fixation
#define etiServerGazeBlink			203	//send new eye tracking blink

//Client "control" messages
#define etiServerNick				205	//send new nickname
#define etiServerDescription		206	//send new description
#define etiServerAdd				207	//send add client to the tracking list
#define etiServerRemove				208	//send remove client from the tracking list

#define etiServerMessage			209	//send message to the client
#define etiServerPoke				210	//send client poke

/* client -> server */
//Gaze Data messages
#define etiClientGazeSample			251	//send new eye tracking sample
#define etiClientGazeFixation		252	//send new eye tracking fixation
#define etiClientGazeBlink			253	//send new eye tracking blink

//Client "control" messages
#define etiClientNick				255	//send new nickname
#define etiClientDescription		256	//send new description
#define etiClientAdd				257	//send add client to the tracking list
#define etiClientRemove				258	//send remove client from the tracking list

#define etiClientMessage			259	//send message
#define etiClientPoke				260	//send poke


/*****************************************************************************
 *
 * Server -> client message definitions
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * etiServerGazeSample - Send gaze data sample.
 *
 * This message is followed by nPoints structures of etiGazeSample structures
 */
typedef struct _etiServerGazeSampleMsg {
    CARD8 type;	// always = etiServerGazeSample
    CARD8 pad1;
	CARD16 id;
	CARD16 pad2;
	CARD16 nPoints;
} etiServerGazeSampleMsg;

#define sz_etiServerGazeSampleMsg 8

/*-----------------------------------------------------------------------------
 * etiServerGazeFixation - Send gaze data fixation.
 *
 * This message is followed by nCount structures of etiGazeFixation structures
 */
typedef struct _etiServerGazeFixationMsg {
    CARD8 type;	// always = etiServerGazeFixation
    CARD8 pad1;
	CARD16 id;
	CARD16 pad2;
	CARD16 nCount;
} etiServerGazeFixationMsg;

#define sz_etiServerGazeFixationMsg 8

/*-----------------------------------------------------------------------------
 * etiServerGazeBlink - Send gaze data blink.
 *
 * This message is followed by nCount structures of etiGazeBlink structures
 */
typedef struct _etiServerGazeBlinkMsg {
    CARD8 type;	// always = etiServerGazeBlink
    CARD8 pad1;
	CARD16 id;
	CARD16 pad2;
	CARD16 nCount;
} etiServerGazeBlinkMsg;

#define sz_etiServerGazeBlinkMsg 8

/*-----------------------------------------------------------------------------
 * etiServerNickMsg - send a client's new nickname that will be shown next to the 
 * eye mark.
 *
 * This message send number of symbols in the nickname and is followed by "length"
 * characters
 */

typedef struct _etiServerNickMsg {
    CARD8 type;			/* always etiServerNick */
	CARD8 length;
	CARD16 id;
} etiServerNickMsg;

#define sz_etiServerNickMsg 4

/*-----------------------------------------------------------------------------
 * etiServerDescriptionMsg - send a client's new description that will be shown 
 * in eye clients options.
 *
 * This message send number of symbols in the description and is followed by "length"
 * characters
 */

typedef struct _etiServerDescriptionMsg {
    CARD8 type;			/* always etiServerDescription */
	CARD8 length;
	CARD16 id;
} etiServerDescriptionMsg;

#define sz_etiServerDescriptionMsg 4

/*-----------------------------------------------------------------------------
 * etiServerAdd - add a new client to the list of trackable marks.
 *
 * This message send the ID of the client to add. This message is followed by 
 * lengthNick + lengthDesc symbols, where lengthNick - length of server nickname, 
 * lengthDesc = length of description
 */
typedef struct _etiServerAddMsg {
    CARD8 type;			/* always etiServerAdd */
	CARD8 pad1;
	CARD16 id;
	CARD8 lengthNick;
	CARD8 lengthDesc;
	CARD16 pad2;
} etiServerAddMsg;

#define sz_etiServerAddMsg 8

/*-----------------------------------------------------------------------------
 * etiServerRemove - remove a client from the list of trackable marks.
 *
 * This message send the ID of the client to remove
 */
typedef struct _etiServerRemoveMsg {
    CARD8 type;			/* always etiServerRemove */
	CARD8 pad1;
	CARD16 id;
} etiServerRemoveMsg;

#define sz_etiServerRemoveMsg 4

/*-----------------------------------------------------------------------------
 * etiServerMessageMsg - send a message to the client.
 *
 * This message send number of symbols in the description and is followed by "length"
 * characters
 */

typedef struct _etiServerMessageMsg {
    CARD8 type;			// always etiServerDescription
	CARD8 pad1;	
	CARD16 pad2;	
	CARD16 id;			//source ID, 0 - means that message directly from the server
	CARD16 length;
} etiServerMessageMsg;

#define sz_etiServerMessageMsg 8

/*-----------------------------------------------------------------------------
 * etiServerPoke - poke the client
 *
 * This message contain an ID of the client which mark should be poked
 */
typedef struct _etiServerPokeMsg {
    CARD8 type;	// always = etiServerPoke
    CARD8 pad;
	CARD16 id;
} etiServerPokeMsg;

#define sz_etiServerPokeMsg 4


/*****************************************************************************
 *
 * Client -> server message definitions
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * etiClientGazeSample - Send gaze data sample.
 *
 * This message is followed by nPoints structures of etiGazeSample structures
 */
typedef struct _etiClientGazeSampleMsg {
    CARD8 type;	// always = etiClientGazeSample
    CARD8 pad;
	CARD16 nPoints;
} etiClientGazeSampleMsg;

#define sz_etiClientGazeSampleMsg 4

/*-----------------------------------------------------------------------------
 * etiClientGazeFixation - Send gaze data fixation.
 *
 * This message is followed by nPoints structures of etiGazeFixation structures
 */
typedef struct _etiClientGazeFixationMsg {
    CARD8 type;	// always = etiClientGazeFixation
    CARD8 pad;
	CARD16 nPoints;
} etiClientGazeFixationMsg;

#define sz_etiClientGazeFixationMsg 4

/*-----------------------------------------------------------------------------
 * etiClientGazeBlink - Send gaze data blink.
 *
 * This message is followed by nPoints structures of etiGazeBlink structures
 */
typedef struct _etiClientGazeBlinkMsg {
    CARD8 type;	// always = etiClientGazeBlink
    CARD8 pad;
	CARD16 nPoints;
} etiClientGazeBlinkMsg;

#define sz_etiClientGazeBlinkMsg 4

/*-----------------------------------------------------------------------------
 * etiClientNickMsg - send a client's new nickname that will be shown next to the 
 * eye mark.
 *
 * This message send number of symbols in the nickname and is followed by "length"
 * characters
 */

typedef struct _etiClientNickMsg {
    CARD8 type;			/* always etiClientNick */
	CARD8 length;
	//followed by *length* chars
} etiClientNickMsg;

#define sz_etiClientNickMsg 2

/*-----------------------------------------------------------------------------
 * etiClientDescriptionMsg - send a client's new description that will be shown 
 * in eye clients options.
 *
 * This message send number of symbols in the description and is followed by "length"
 * characters
 */

typedef struct _etiClientDescriptionMsg {
    CARD8 type;			/* always etiClientDescription */
	CARD8 length;
	//followed by *length* chars
} etiClientDescriptionMsg;

#define sz_etiClientDescriptionMsg 2

/*-----------------------------------------------------------------------------
 * etiClientAdd - add a new client to the list of trackable marks.
 *
 * This message send the ID of the client to add, length of the nickname and length of description.
 * Its followed by lengthNick characters of client nickname and lengthDesc characters of client description
 */
typedef struct _etiClientAddMsg {
    CARD8 type;			/* always etiClientAdd */
	CARD8 pad1;
	CARD8 lengthNick;
	CARD8 lengthDesc;
} etiClientAddMsg;

#define sz_etiClientAddMsg 4

/*-----------------------------------------------------------------------------
 * etiClientRemove - remove a client from the list of trackable marks.
 *
 * This message send the ID of the client to remove
 */
typedef struct _etiClientRemoveMsg {
    CARD8 type;			/* always etiClientRemove */
} etiClientRemoveMsg;

#define sz_etiClientRemoveMsg 1

/*-----------------------------------------------------------------------------
 * etiClientMessageMsg - send a message to the client.
 *
 * This message send number of symbols in the description and is followed by "length"
 * characters
 */

typedef struct _etiClientMessageMsg {
    CARD8 type;			// always etiClientMessage
	CARD8 pad;
	CARD16 length;
} etiClientMessageMsg;

#define sz_etiClientMessageMsg 4

/*-----------------------------------------------------------------------------
 * etiClientPoke - poke the client
 *
 * This message contain an ID of the client which mark should be poked
 */
typedef struct _etiClientPokeMsg {
    CARD8 type;	// always = etiClientPoke
} etiClientPokeMsg;

#define sz_etiClientPokeMsg 1
