/*-----------------------------------------------------------------------------
 * This file defines custom data type, used to store and process gaze data
 */
#pragma once

#include "..\winvnc\rfb.h"
#include <vector>
#include <string>
#include <ostream>

//Gaze Data Items' types
typedef enum _etiGazeDataElementType
{
	GDT_UNDEFINED,		
	GDT_SAMPLE,			
	GDT_SINGLESAMPLE,
	GDT_FIXATION,
	GDT_BLINK,
	GDT_EVENT			
} etiGazeDataElementType;

//Gaze Data Items' types
typedef enum _etiLogRecordType
{
	LRT_UNDEFINED,		
	LRT_GAZEDATA,
	LRT_MESSAGE,
	LRT_ADDCLIENT,
	LRT_REMCLIENT,
	LRT_POKECLIENT,
	LRT_TRACKSTART,
	LRT_SETNICK,
	LRT_SETDESC,
	LRT_OTHER
} etiLogRecordType;


/****************************************************************************/
/*	Protocol messages ETI data types										*/
/****************************************************************************/

/**
 * Sample gaze data for one eye.
 */
typedef struct _etiGazeSingleSample
{
	CARD32 number;		//ordinal number
	CARD32 time;		//time offset from the start of the record
	CARD16 eye;		//event code
	CARD32 x;		//x coordinate of the eye
	CARD32 y;		//y coordinate of the eye
	CARD32 pupil;	//pupil size
} etiGazeSingleSample;

#define sz_etiGazeSingleSample 22

/**
 * Fixation gaze data.
 */
typedef struct _etiGazeFixation
{
	CARD32 number;		//ordinal number
	CARD32 time;		//time offset from the start of the record
	CARD32 duration;	//fixation duration
	CARD16 eye;			//eye number
	CARD32 x;			//x coordinate of the eye
	CARD32 y;			//y coordinate of the eye
} etiGazeFixation;

#define sz_etiGazeFixation 22

/**
 * Blink gaze data.
 */
typedef struct _etiGazeBlink
{
	CARD32 number;		//ordinal number
	CARD32 time;		//time offset from the start of the record
	CARD32 duration;	//fixation duration
	CARD16 eye;			//eye number
} etiGazeBlink;

#define sz_etiGazeBlink 8

struct etiTimeStamp
{
	short Year;	//current year
	int Days;	//days past since January 1st
	unsigned long Milliseconds;	//Milliseconds past since 00:00 for the current day
};

