#pragma once

#include <list>

#include <boost\shared_ptr.hpp>

#include "entries\CGDBlink.h"
#include "entries\CGDFixation.h"
#include "entries\CGDSingleSample.h"

using namespace std;

template<class T>
class GDLIBAPI CGDQueue
{
private:
	unsigned int m_maxelements;
	list<T> m_elements;

public:
	CGDQueue(int maxLength):
	  m_maxelements(maxLength), m_elements()
	{	}

	void clear()
	{
		if (m_elements.size() > 0)
			m_elements.clear();
	}

	void add(T e)
	{
		while(m_elements.size() >= m_maxelements)
			m_elements.pop_front();

		m_elements.push_back(e);
	}

	const T& get()
	{
		return m_elements.begin().get();
	}

	T& pop()
	{
		return m_elements.pop_front().get();
	}
};

typedef boost::shared_ptr<CGDQueue<CGDBlinkPtr>> CGDBlinksQueuePtr;
typedef boost::shared_ptr<CGDQueue<CGDFixationPtr>> CGDFixationsQueuePtr;
typedef boost::shared_ptr<CGDQueue<CGDSingleSamplePtr>> CGDSSampleQueuePtr;

