#pragma once

#include <list>

#include <boost\shared_ptr.hpp>

#include "gdtypes.h"
#include "GDEntry.h"

using namespace std;

template<class T>
class CGDEntriesQueue
{
private:
	unsigned int m_maxelements;
	list<boost::shared_ptr<T>> m_elements;

public:
	CGDEntriesQueue(int maxLength):
	  m_maxelements(maxLength)
	{	}

	void clear()
	{
		if (m_elements.size() > 0)
			m_elements.clear();
	}

	void add(T* e)
	{
		while(m_elements.size() >= m_maxelements)
			m_elements.pop_front();

		m_elements.push_back(boost::shared_ptr<T>(e));
	}

	const T& get()
	{
		return m_elements.begin().get();
	}

	T& pop()
	{
		return m_elements.pop_front().get();
	}
};

typedef boost::shared_ptr<CGDEntriesQueue<GDBlink>> CGDBlinksQueuePtr;
typedef boost::shared_ptr<CGDEntriesQueue<GDFixation>> CGDFixationsQueuePtr;
typedef boost::shared_ptr<CGDEntriesQueue<GDSingleSample>> CGDSSampleQueuePtr;

