#include "CGDLogger.h"

/*******************************************************************************/

CGDLogger::CGDLogger(string outFolder):
  mOutputFolder(outFolder) 
{ 
	mFiller = "==========================================================================================================="; 
}

/*******************************************************************************/

CGDLogger::~CGDLogger()
{
	close();
}

/*******************************************************************************/

void CGDLogger::open(string fileName, vncClientId cId)
{
	if (!mOutStream.is_open() &&
		fileName != "")
	{
		mId = cId;

		mFileName = fileName;
		mOutStream.open(mOutputFolder + "\\" + mFileName, ios_base::out);

		printHeader();
	}

	return;
}

/*******************************************************************************/

void CGDLogger::close()
{
	if (mOutStream.is_open())
	{
		printFooter();

		mRecStartToken.clear();
		mOutStream.flush();
		mOutStream.close();
	}
}

/*******************************************************************************/

void CGDLogger::print(std::stringstream& ss, bool printTime)
{
	if (isReady() && ss.tellg()>=0)
	{
		if (printTime)
			mOutStream << TimeStamp() << ";";

		mOutStream << ss.str() << endl;
		mOutStream.flush();

		mCount++;
	}
}

/*******************************************************************************/

void CGDLogger::printHeader()
{
	std::stringstream logheader;

	logheader << "Gaze data log folder: " << mOutputFolder << endl;
	logheader << "Gaze data log file: " << mFileName << endl;
	logheader << "Gaze client: " << mId << endl;
	logheader << "Started at " << TimeStamp() << endl;
	logheader << endl;
	logheader << mFiller << endl;
	logheader << endl; 

	logheader << "LOGRECTIME" << ";";
	logheader << "LOGRECORDTYPE" << ";";
	logheader << "SOURCECLIENTID" << ";";
	logheader << "CLIENTRECSTARTTIME" << ";";
	logheader << "DATA" << ";";
	logheader << "<ERROR>";

	print(logheader, false);
}

/*******************************************************************************/

void CGDLogger::printFooter()
{
	std::stringstream logfooter;
	
	logfooter << endl; 
	logfooter << mFiller << endl;
	logfooter << endl;
	logfooter << "Gaze data log records: " << mCount << endl;
	logfooter << "Ended at " << TimeStamp() << endl;

	print(logfooter);
}

/*******************************************************************************/

void CGDLogger::setRecordStartToken(TimeStamp recordstart)
{
	mRecStartToken.clear();
	mRecStartToken << recordstart;
}

/*******************************************************************************/

bool CGDLogger::isReady()
{
	return mOutStream.is_open() && mOutStream.good();
}

/**Log received message*/
void CGDLogger::log(gdLogRecordType logType, string msg, string err)
{
	/**
	 * LOGRECTIME LOGRECORDTYPE SOURCECLIENTID CLIENTRECSTARTTIME DATA <ERROR> 
     */
	stringstream data;

	switch(logType)
	{
	case LRT_GAZEDATA:
	case LRT_MESSAGE:
	case LRT_REGCLIENT:
	case LRT_UNREGCLIENT:
	case LRT_POKECLIENT:
	case LRT_STARTTRACK:
	case LRT_SETNICKNAME:
	case LRT_SETDESCRIPT:
	case LRT_UNDEFINED:
		data << logType << ";";
		data << mId << ";";
		data << mRecStartToken.str() << ";";
		data << msg << ";";
		if (!err.empty())
			data << err;
		break;
	default:
		return;
	}

	print(data);
}

/*******************************************************************************/