#include "CGDVncClient.h"
#include "gdproto.h"
#include "winvnc\vncServer.h"

/*******************************************************************************/

CGDVncClient::CGDVncClient(int maxBlinks, int maxSamples, int maxFixations, std::string logFolder):
	mLog(logFolder), mRegistered(false)
{
	mBlinks = CGDBlinksQueuePtr(new CGDEntriesQueue<GDBlink>(maxBlinks));
	mLeftSamples = new CGDEntriesQueue<GDSingleSample>(maxSamples);
	mRightSamples = new CGDEntriesQueue<GDSingleSample>(maxSamples);
	mFixations = CGDFixationsQueuePtr(new CGDEntriesQueue<GDFixation>(maxFixations));

	//initLogFile();
}

/*******************************************************************************/

CGDVncClient::~CGDVncClient()
{
	if (mBlinks)
	{
		mBlinks->clear();
		mBlinks.reset();
	}

	if (mLeftSamples)
	{
		mLeftSamples->clear();
		delete mLeftSamples;
		mLeftSamples = NULL;
	}

	if (mRightSamples)
	{
		mRightSamples->clear();
		delete mRightSamples;
		mRightSamples = NULL;
	}

	if (mFixations)
	{
		mFixations->clear();
		mFixations.reset();
	}

	if (mLog.isReady())
	{
		mLog.close();
	}
}

/*******************************************************************************/

void CGDVncClient::initLogFile()
{
	if (mLog.isReady())
		return;

	ostringstream fileName;
	fileName << "GazeLog_" << getLogFileMarker() << "_at" << TimeStamp() << ".log";

	try
	{
		mLog.open(fileName.str(), m_id);
	}
	catch(std::exception ex){ }
}

BOOL CGDVncClient::sendHeaderMessage(CARD8 type, CARD8 subtype, int srcId, int cnt)
{
	gdHdrMsg header;
	header.type = type;
	header.subtype = subtype;
	header.id = Swap16IfLE(srcId);
	header.count = Swap16IfLE(cnt);

	return SendRFBMsg(type, (BYTE*) &header, sz_gdHdrMsg);
}

BOOL CGDVncClient::sendText(std::string text)
{
	if (!m_socket->SendQueued((char *) text.c_str(), text.length()))
	{
		Kill();
		return false;
	}

	return true;
}

template<class T>
BOOL CGDVncClient::sendGazeEntry(std::list<T> entries, int size)
{
	//send structs
	for(std::list<T>::iterator it = entries.begin(); it != entries.end(); it++)
	{
		if (!m_socket->SendQueued((char *) &(*it), size))
		{
			Kill();
			return false;
		}
	}
	
	return true;
}

/*******************************************************************************/

std::string CGDVncClient::getLogFileMarker()
{
	stringstream ss;
	ss << m_id;
	
	return ((ss.tellp() < 1) ? "[none]" : ss.str());
}

/*******************************************************************************/

void CGDVncClient::setNickname(std::string name)
{
	mNickname = name;
}

/*******************************************************************************/

void CGDVncClient::setDescription(std::string description)
{
	mDescription = description;
}

/*******************************************************************************/

void CGDVncClient::setRegistered(BOOL reg)
{
	mRegistered = reg;
}

/*******************************************************************************/

std::string CGDVncClient::getNickname()
{
	return mNickname.value();
}

/*******************************************************************************/

std::string CGDVncClient::getDescription()
{
	return mDescription.value();
}

/*******************************************************************************/

BOOL CGDVncClient::isRegistered()
{
	return mRegistered;
}

/*******************************************************************************/

/*template<typename T>
BOOL CGDVncClient::sendGDMessage(CARD8 msgSubType, vncClientId id, int structSize, std::list<T> entries)
{
	if (entries.size() < 1 || id == m_id)
		return false;

	gdServerHdrMsg header;
	header.count = Swap16IfLE(entries.size());
	header.subtype = msgSubType;
	header.id = Swap16IfLE(id);

	omni_mutex_lock l(m_sendUpdateLock);

	//send header with the number of following entries structs
	if (!SendRFBMsg(gdGazeMsg, (BYTE*) &header, sz_gdServerHdrMsg))
	{
		return false;
	}

	//send structs
	for(std::list<T>::iterator it = entries.begin(); it != entries.end(); it++)
	{
		if (!m_socket->SendQueued((char *) &(*it), structSize))
		{
			Kill();
			return false;
		}
	}
	
	return true;
}

/*******************************************************************************/

/*BOOL CGDVncClient::sendTextMessage(CARD8 msgSubType, vncClientId id, std::string text)
{
	if (text.length() < 1 || id == m_id)
		return false;

	if (text.length() > 256)
		text = text.substr(0, 256);

	gdServerHdrMsg header;
	header.id = Swap16IfLE(id);
	header.subtype = msgSubType;
	header.count = text.length();
	
	omni_mutex_lock l(m_sendUpdateLock);

	//send header with the number of following characters
	if (!SendRFBMsg(gdTextMsg, (BYTE*) &header, sz_gdServerHdrMsg))
	{
		return false;
	}

	if (!m_socket->SendQueued((char *) text.c_str(), text.length()))
	{
		Kill();
		return false;
	}

	return true;
}

/*******************************************************************************/

/*BOOL CGDVncClient::sendIdMessage(CARD8 msgSubType, vncClientId id)
{
	if (m_id == id)
		return false;

	gdServerHdrMsg header;
	header.id = Swap16IfLE(id);
	header.subtype = msgSubType;
	header.count = 0;
	
	omni_mutex_lock l(m_sendUpdateLock);

	//send header with the number of following characters
	if (!SendRFBMsg(gdControlMsg, (BYTE*) &header, sz_gdServerHdrMsg))
	{
		return false;
	}
	
	return true;
}*/

/*******************************************************************************/

BOOL CGDVncClient::sendGDBlink(vncClientId id, std::list<gdBlink> blinks)
{
	if (blinks.size() < 1 || id == m_id)
		return false;

	omni_mutex_lock l(m_sendUpdateLock);
	if (!sendHeaderMessage(gdGazeMsg, gdGazeBlink, id, blinks.size()))
		return false;

	return sendGazeEntry(blinks, sz_gdBlink);
}

/*******************************************************************************/

BOOL CGDVncClient::sendGDSingleSample(vncClientId id, std::list<gdSingleSample> samples)
{
	if (samples.size() < 1 || id == m_id)
		return false;

	omni_mutex_lock l(m_sendUpdateLock);
	if (!sendHeaderMessage(gdGazeMsg, gdGazeSample, id, samples.size()))
		return false;

	return sendGazeEntry(samples, sz_gdSingleSample);
}

/*******************************************************************************/

BOOL CGDVncClient::sendGDFixation(vncClientId id, std::list<gdFixation> fixations)
{
	if (fixations.size() < 1 || id == m_id)
		return false;

	omni_mutex_lock l(m_sendUpdateLock);
	if (!sendHeaderMessage(gdGazeMsg, gdGazeFixation, id, fixations.size()))
		return false;

	return sendGazeEntry(fixations, sz_gdFixation);
}

/*******************************************************************************/

BOOL CGDVncClient::sendNickname(vncClientId id, std::string nn)
{
	if (nn.length() < 1 || id == m_id)
		return false;

	omni_mutex_lock l(m_sendUpdateLock);

	if (!sendHeaderMessage(gdTextMsg, gdName, id, nn.length()))
		return false;

	return sendText(nn);
}

/*******************************************************************************/

BOOL CGDVncClient::sendDescription(vncClientId id, std::string dscr)
{
	if (dscr.length() < 1 || id == m_id)
		return false;

	omni_mutex_lock l(m_sendUpdateLock);

	if (!sendHeaderMessage(gdTextMsg, gdDescription, id, dscr.length()))
		return false;

	return sendText(dscr);
}

/*******************************************************************************/

BOOL CGDVncClient::sendTextMessage(vncClientId id, std::string text)
{
	if (text.length() < 1 || id == m_id)
		return false;

	omni_mutex_lock l(m_sendUpdateLock);

	if (!sendHeaderMessage(gdTextMsg, gdText, id, text.length()))
		return false;

	return sendText(text);
}

/*******************************************************************************/

BOOL CGDVncClient::sendRegistration(vncClientId id)
{
	if (id == m_id)
		return false;

	omni_mutex_lock l(m_sendUpdateLock);

	if (!sendHeaderMessage(gdControlMsg, gdRegister, id, 0))
		return false;

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::sendUnRegistration(vncClientId id)
{
	if (id == m_id)
		return false;

	omni_mutex_lock l(m_sendUpdateLock);

	if (!sendHeaderMessage(gdControlMsg, gdUnregister, id, 0))
		return false;

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::sendPoke(vncClientId id)
{
	if (id == m_id)
		return false;

	omni_mutex_lock l(m_sendUpdateLock);

	if (!sendHeaderMessage(gdControlMsg, gdPoke, id, 0))
		return false;

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::receiveGDSingleSample(std::list<gdSingleSample> samples)
{
	string err;
	
	for(std::list<gdSingleSample>::iterator it = samples.begin(); it != samples.end(); it++)
	{
		err = "";

		GDSingleSample* smpl = new GDSingleSample(*it);

		switch(smpl->eye())
		{
		case GDE_LEFT: //left eye
			mLeftSamples->add(smpl);
			break;
		case GDE_RIGHT: //right eye
			mRightSamples->add(smpl);
			break;
		default:
			err = "Unrecognized 'Eye'-field value";
			break;
		}

		mLog.log(LRT_GAZEDATA, smpl->toString(), err);
	}

	m_server->distGDSingleSample(m_id, samples);

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::receiveGDBlink(std::list<gdBlink> blinks)
{
	string err;
	
	for(std::list<gdBlink>::iterator it = blinks.begin(); it != blinks.end(); it++)
	{
		err = "";

		GDBlink *blnk = new GDBlink(*it);

		mBlinks->add(blnk);

		mLog.log(LRT_GAZEDATA, blnk->toString(), err);
	}

	m_server->distGDBlink(m_id, blinks);

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::receiveGDFixation(std::list<gdFixation> fixations)
{
	string err;
	
	for(std::list<gdFixation>::iterator it = fixations.begin(); it != fixations.end(); it++)
	{
		err = "";

		GDFixation *fxn = new GDFixation(*it);

		mFixations->add(fxn);

		mLog.log(LRT_GAZEDATA, fxn->toString(), err);
	}

	m_server->distGDFixation(m_id, fixations);

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::receiveNickname(std::string nn)
{
	mLog.log(LRT_SETNICKNAME, mNickname.value());

	setNickname(nn);

	m_server->distNickname(m_id, nn);

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::receiveDescription(std::string nd)
{
	mLog.log(LRT_SETDESCRIPT, mDescription.value());

	setDescription(nd);
	
	m_server->distDescription(m_id, nd);

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::receiveTextMessage(std::string msg)
{
	mLog.log(LRT_MESSAGE, msg);
	
	m_server->distTextMessage(m_id, msg);

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::receiveRegistration()
{
	if (!isRegistered())
	{
		mLog.log(LRT_REGCLIENT, "Register client");
		mRegistered = true;

		mLog.log(LRT_STARTTRACK, "Tracking restarted");
		mRecStart = TimeStamp();
		mLog.setRecordStartToken(mRecStart);

		m_server->distRegistration(m_id);
	}

	return true;
}

BOOL CGDVncClient::receiveUnRegistration()
{
	if (isRegistered())
	{
		mRegistered = false;
		mLog.log(LRT_UNREGCLIENT, "Unregister client");

		m_server->distUnRegistration(m_id);
	}

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::receivePoke()
{
	if (isRegistered())
	{
		mRegistered = false;
		mLog.log(LRT_UNREGCLIENT, "Unregister client");

		m_server->distPoke(m_id);
	}

	return true;
}

/*******************************************************************************/
/*
#pragma region RECEIVING data
	BOOL ReceiveRegRequest(std::string, std::string);
	BOOL ReceivePoke();
#pragma endregion
*/

//TODO:
/**
Receiving packs of entries:
* receive several of entries
* try to add all of them to the queue
* distribute the last non-error entry
*/