#pragma once

#include "../winvnc/stdhdrs.h"
#include "../winvnc/VSocket.h"
#include "../winvnc/rfb.h"
#include <omnithread.h>

#include <list>

#include "gdtypes.h"

#include "CGDTextParameter.h"
#include "CGDLogger.h"
#include "GDEntry.h"
#include "CGDEntriesQueue.h"

class vncServer;

typedef short vncClientId;

class CGDVncClient
{
public:
	//CGDVncClient(): mLog("") { } //todo: finalize log folder specification
	CGDVncClient(int maxBlinks, int maxSamples, int maxFixations, std::string logFolder);
	virtual ~CGDVncClient();

public:
	void test();
	void initLogFile();
protected:
	std::string getLogFileMarker();

	BOOL sendHeaderMessage(CARD8 type, CARD8 subtype, int srcId, int cnt);
	BOOL sendText(std::string text);
	template<class T>
	BOOL sendGazeEntry(std::list<T>, int);

//Getters/setters
public:
	void setNickname(std::string name);
	void setDescription(std::string description);
	void setRegistered(BOOL reg);

	std::string getNickname();
	std::string getDescription();
	BOOL isRegistered();

//information handling
public:
#pragma region SENDING data to the client
	BOOL sendGDSingleSample(vncClientId, std::list<gdSingleSample>);
	BOOL sendGDBlink(vncClientId, std::list<gdBlink>);
	BOOL sendGDFixation(vncClientId, std::list<gdFixation>);

	BOOL sendTextMessage(vncClientId, std::string);
	BOOL sendNickname(vncClientId, std::string);
	BOOL sendDescription(vncClientId, std::string);

	BOOL sendRegistration(vncClientId);
	BOOL sendUnRegistration(vncClientId);
	BOOL sendPoke(vncClientId);
#pragma endregion

#pragma region RECEIVING data from the client
	BOOL receiveGDSingleSample(std::list<gdSingleSample>);
	BOOL receiveGDBlink(std::list<gdBlink>);
	BOOL receiveGDFixation(std::list<gdFixation>);

	BOOL receiveNickname(std::string);
	BOOL receiveDescription(std::string);
	BOOL receiveTextMessage(std::string);

	BOOL receiveRegistration();
	BOOL receiveUnRegistration();
	BOOL receivePoke();
	
	BOOL receiveStartTracking();
#pragma endregion

protected:
	virtual BOOL SendRFBMsg(CARD8 type, BYTE *buffer, int buflen) = 0;
	virtual void Kill() = 0;

protected:
	//base project compatability
	vncServer		*m_server;
	vncClientId		m_id;
	omni_mutex		m_sendUpdateLock;
	VSocket			*m_socket;

private:
	BOOL mRegistered;
	CGDTextParameter<MAX_NICKNAME_LENGTH> mNickname;
	CGDTextParameter<MAX_DESCRIPTION_LENGTH> mDescription;

	CGDBlinksQueuePtr mBlinks;
	//CGDSSampleQueuePtr mLeftSamples;
	//CGDSSampleQueuePtr mRightSamples;
	CGDEntriesQueue<GDSingleSample>* mLeftSamples;
	CGDEntriesQueue<GDSingleSample>* mRightSamples;
	CGDFixationsQueuePtr mFixations;

	TimeStamp mRecStart;

	CGDLogger mLog;
};

