#pragma once

#include <string>
#include <sstream>

#include <boost\shared_ptr.hpp>
#include "gdtypes.h"
#include "IGDEntry.h"

template<typename T>
class GDEntry : public IGDEntry<T>
{ };

/****************************************************************************/
template<>
class GDEntry<gdBlink>:
	public IGDEntry<gdBlink>
{
public:
	GDEntry(gdBlink entry):IGDEntry<gdBlink>(entry) { }
	virtual ~GDEntry() { }

	virtual gdElementType type(){ return GDT_BLINK; }
	virtual long duration(){ return mData.duration; }
	virtual short eye(){ return mData.eye; }

	virtual string toString()
	{
		stringstream stream;

		stream << time() << "\t";
		stream << number() << "\t";
		stream << -1 << "\t";
		stream << -1 << "\t";
		stream << duration() << "\t";
		stream << eye() << "\t";
		stream << -1 << "\t";
		stream << type() << "\t";
  
		return stream.str();
	}
};

typedef GDEntry<gdBlink> GDBlink;
typedef boost::shared_ptr<GDBlink> GDBlinkPtr;
/****************************************************************************/

/****************************************************************************/
template<>
class GDEntry<gdFixation>:
	public IGDEntry<gdFixation>
{
public:
	GDEntry(gdFixation entry):IGDEntry<gdFixation>(entry) { }
	virtual ~GDEntry() { }

	virtual gdElementType type(){ return GDT_FIXATION; }
	virtual long duration(){ return mData.duration; }
	virtual short eye(){ return mData.eye; }
	virtual long x(){ return mData.x; }
	virtual long y(){ return mData.y; }

	virtual string toString()
	{
		stringstream stream;

		stream << time() << "\t";
		stream << number() << "\t";
		stream << x() << "\t";
		stream << y() << "\t";
		stream << duration() << "\t";
		stream << eye() << "\t";
		stream << -1 << "\t";
		stream << type() << "\t";
  
		return stream.str();
	}
};

typedef GDEntry<gdFixation> GDFixation;
typedef boost::shared_ptr<GDFixation> GDFixationPtr;
/****************************************************************************/

/****************************************************************************/
template<>
class GDEntry<gdSingleSample>:
	public IGDEntry<gdSingleSample>
{
public:
	GDEntry(gdSingleSample entry):IGDEntry<gdSingleSample>(entry) { }
	virtual ~GDEntry() { }

	virtual gdElementType type(){ return GDT_FIXATION; }
	virtual long pupil(){ return mData.pupil; }
	virtual short eye(){ return mData.eye; }
	virtual long x(){ return mData.x; }
	virtual long y(){ return mData.y; }

	virtual string toString()
	{
		stringstream stream;

		stream << time() << "\t";
		stream << number() << "\t";
		stream << x() << "\t";
		stream << y() << "\t";
		stream << -1 << "\t";
		stream << eye() << "\t";
		stream << pupil() << "\t";
		stream << type() << "\t";
  
		return stream.str();
	}
};

typedef GDEntry<gdSingleSample> GDSingleSample;
typedef boost::shared_ptr<GDSingleSample> GDSingleSamplePtr;
/****************************************************************************/
