//#pragma once
#ifndef _CGAZECLIENT_H_
#define _CGAZECLIENT_H_

#include <boost/shared_ptr.hpp>

#include "etiextension.h"
#include <iostream>
#include <fstream>
#include <sstream>

#include <map> 

#include "CGazeDataEntryFixation.h"
#include "CGazeDataEntryBlink.h"
#include "CGazeDataEntrySingleSample.h"


using namespace boost;

class CGazeClient
{
private:
	//eti client specific data
	std::string m_nickname;
	std::string m_description;
	short m_id;

	//length of gaze data vectors. Maximum amount of gaze data 
	//entries stored in the client
	unsigned int m_blinkQueueLength;
	unsigned int m_fixationQueueLength;
	unsigned int m_sampleQueueLength;

	//vectors of eye gaze entries
	etiGazeBlinkVector m_blinks;
	etiGazeFixationVector m_fixations;
	etiGazeSampleVector m_leftEyeSamples;
	etiGazeSampleVector m_rightEyeSamples;

	//timestamp of the last received "record start" message. 
	//It is used to calculate "absolute" timestamp of the gaze data entry
	etiTimeStamp m_recordStart; 

	//client specific log files
	CGazeLoggerPtr m_blinksLog;
	CGazeLoggerPtr m_fixationsLog;
	CGazeLoggerPtr m_singleSamplesLog;
		
private:
	void ClearGazeDataVectors();
	void InitializeLogFiles(short, std::string);
	void DeInitializeLogFiles();
		
public:
	CGazeClient(int, int, int, short, std::string);
	~CGazeClient(void);

	void TrackingStarted();

	void AddEyeSample(CGazeDataEntrySingleSamplePtr etiItem);
	void AddBlink(CGazeDataEntryBlinkPtr etiItem);
	void AddFixation(CGazeDataEntryFixationPtr etiItem);

	void SetNick(std::string nick);
	void SetDesc(std::string desc);

	std::string GetNick();
	std::string GetDesc();
};	

typedef boost::shared_ptr<CGazeClient> CGazeClientPtr;
typedef std::map<short, CGazeClientPtr> etiClientsMap;

#endif