#pragma once

#include <iostream>
#include <fstream>
#include <sstream>

#include <boost\shared_ptr.hpp>

#include ".\ETI\etitypes.h"
#include "etiextension.h"
#include "CGazeDataLogger.h"
#include "CGazeDataEntryBlink.h"
#include "CGazeDataEntryFixation.h"
#include "CGazeDataEntrySingleSample.h"

class CGazeDataClient
{
private:
	//eti client specific data
	std::string m_nickname;
	std::string m_description;
	short m_id;

	//length of gaze data vectors. Maximum amount of gaze data 
	//entries stored in the client
	unsigned int m_blinkQueueLength;
	unsigned int m_fixationQueueLength;
	unsigned int m_sampleQueueLength;

	//vectors of eye gaze entries
	etiGazeBlinkVector m_blinks;
	etiGazeFixationVector m_fixations;
	etiGazeSampleVector m_leftEyeSamples;
	etiGazeSampleVector m_rightEyeSamples;

	//timestamp of the last received "record start" message. 
	//It is used to calculate "absolute" timestamp of the gaze data entry
	etiTimeStamp m_recordStart; 

	//client specific log files
	CGazeDataLoggerPtr m_blinksLog;
	CGazeDataLoggerPtr m_fixationsLog;
	CGazeDataLoggerPtr m_singleSamplesLog;

private:
	void ClearGazeDataVectors();
	void InitializeLogFiles(short, std::string);
	void DeInitializeLogFiles();

public:
	CGazeDataClient(int, int, int, short, std::string);
	~CGazeDataClient();

	void TrackingStarted();

	void AddEyeSample(CGazeDataEntrySingleSamplePtr etiItem);
	void AddBlink(CGazeDataEntryBlinkPtr etiItem);
	void AddFixation(CGazeDataEntryFixationPtr etiItem);

	void SetNick(std::string nick);
	void SetDesc(std::string desc);

	std::string GetNick();
	std::string GetDesc();

	void SetFixationsQueueLength(int);
	void SetBlinksQueueLength(int);
	void SetSamplesQueueLength(int);
};

typedef boost::shared_ptr<CGazeDataClient> CGazeDataClientPtr;
typedef std::map<short, CGazeDataClientPtr> etiGDClientsMap;


