#include "..\vncClient.h"

BOOL vncClient::SendAddClient(vncClientId id, std::string nickname, std::string description)
{
	etiServerAddMsg addClientMsg;

	addClientMsg.id = Swap16IfLE(id);
	addClientMsg.lengthNick = Swap32IfLE(nickname.length());
	addClientMsg.lengthDesc = Swap32IfLE(description.length());
	
	omni_mutex_lock l(m_sendUpdateLock);

	//send message header
	if (!SendRFBMsg(etiServerAdd, (BYTE *) &addClientMsg, sz_etiServerAddMsg))
	{
		return false;
	}
	//send nickname
	if (!m_socket->SendQueued(nickname.c_str(), nickname.length()))
	{
		return false;
	}
	//send description
	if (!m_socket->SendQueued(description.c_str(), description.length()))
	{
		return false;
	}

	return true;
}

BOOL vncClient::SendRemoveClient(vncClientId id)
{
	etiServerRemoveMsg removeClientMsg;

	removeClientMsg.id = Swap16IfLE(id);
	
	omni_mutex_lock l(m_sendUpdateLock);

	//send message header
	if (!SendRFBMsg(etiServerRemove, (BYTE *) &removeClientMsg, sz_etiServerRemoveMsg))
	{
		return false;
	}

	return true;
}

BOOL vncClient::SendClientMessage(vncClientId id, std::string message)
{
	etiServerMessageMsg msgClientMsg;

	msgClientMsg.id = Swap16IfLE(id);
	msgClientMsg.length = Swap32IfLE(message.length());
	
	omni_mutex_lock l(m_sendUpdateLock);

	//send message header
	if (!SendRFBMsg(etiServerMessage, (BYTE *) &msgClientMsg, sz_etiServerMessageMsg))
	{
		return false;
	}
	//send message
	if (!m_socket->SendQueued(message.c_str(), message.length()))
	{
		return false;
	}

	return true;
}

BOOL vncClient::SendPoke(vncClientId id)
{
	etiServerPokeMsg pokeClientMsg;

	pokeClientMsg.id = Swap16IfLE(id);
	
	omni_mutex_lock l(m_sendUpdateLock);

	//send message header
	if (!SendRFBMsg(etiServerPoke, (BYTE *) &pokeClientMsg, sz_etiServerPokeMsg))
	{
		return false;
	}

	return true;
}

BOOL vncClient::SendClientNick(vncClientId id, std::string nickname)
{
	etiServerNickMsg nickClientMsg;

	//addClientMsg.type = etiServerAdd;
	nickClientMsg.id = Swap16IfLE(id);
	nickClientMsg.length = Swap32IfLE(nickname.length());
	
	omni_mutex_lock l(m_sendUpdateLock);

	//send message header
	if (!SendRFBMsg(etiServerNick, (BYTE *) &nickClientMsg, sz_etiServerNickMsg))
	{
		return false;
	}
	//send nickname
	if (!m_socket->SendQueued(nickname.c_str(), nickname.length()))
	{
		return false;
	}

	return true;
}

BOOL vncClient::SendClientDescription(vncClientId id, std::string description)
{
	etiServerDescriptionMsg descClientMsg;

	//addClientMsg.type = etiServerAdd;
	descClientMsg.id = Swap16IfLE(id);
	descClientMsg.length = Swap32IfLE(description.length());
	
	omni_mutex_lock l(m_sendUpdateLock);

	//send message header
	if (!SendRFBMsg(etiServerDescription, (BYTE *) &descClientMsg, sz_etiServerDescriptionMsg))
	{
		return false;
	}
	//send nickname
	if (!m_socket->SendQueued(description.c_str(), description.length()))
	{
		return false;
	}

	return true;
}

BOOL vncClient::SendGazeSingleSample(vncClientId id, etiGazeSingleSample gazeSample)
{
	etiServerGazeSampleMsg gazeSampleClientMsg;

	gazeSampleClientMsg.id = Swap16IfLE(id);
	gazeSampleClientMsg.nPoints = Swap16IfLE(1);

	omni_mutex_lock l(m_sendUpdateLock);

	//send message header
	if (!SendRFBMsg(etiServerGazeSample, (BYTE *) &gazeSampleClientMsg, sz_etiServerGazeSampleMsg))
	{
		return false;
	}
	//send sample
	if (!m_socket->SendQueued((char *) &gazeSample, sz_etiGazeSingleSample))
	{
		return false;
	}

	return true;
}

BOOL vncClient::SendGazeBlink(vncClientId id, etiGazeBlink blinkSample)
{
	etiServerGazeBlinkMsg gazeBlinkMsg;

	gazeBlinkMsg.id = Swap16IfLE(id);
	gazeBlinkMsg.nCount = Swap16IfLE(1);

	omni_mutex_lock l(m_sendUpdateLock);

	//send message header
	if (!SendRFBMsg(etiServerGazeBlink, (BYTE *) &gazeBlinkMsg, sz_etiServerGazeBlinkMsg))
	{
		return false;
	}
	//send sample
	if (!m_socket->SendQueued((char *) &blinkSample, sz_etiGazeBlink))
	{
		return false;
	}

	return true;
}

BOOL vncClient::SendGazeFixation(vncClientId id, etiGazeFixation fixationSample)
{
	etiServerGazeFixationMsg gazeFixationMsg;

	gazeFixationMsg.id = Swap16IfLE(id);
	gazeFixationMsg.nCount = Swap16IfLE(1);

	omni_mutex_lock l(m_sendUpdateLock);

	//send message header
	if (!SendRFBMsg(etiServerGazeFixation, (BYTE *) &gazeFixationMsg, sz_etiServerGazeFixationMsg))
	{
		return false;
	}
	//send sample
	if (!m_socket->SendQueued((char *) &fixationSample, sz_etiGazeFixation))
	{
		return false;
	}

	return true;
}
