/*
 * etiproto.h - header file defining extension of RFB protocol for the purposes 
 * of eye-tracking information processing.
 *
 * message name format
 * [marker][origin[description]
 * 
 * [marker] - always "gd"
 * [origin] - "server" or "client"
 * [description] - short representation of that what this message is intented for
 */

#pragma once

#include <map>

//Define CARD* as in rfb.h
typedef unsigned long CARD32;
typedef unsigned short CARD16;
typedef short INT16;
typedef unsigned char  CARD8;


#define SERVER_ID 65000
#define ANY_ID 64000

/*****************************************************************************
 *
 * Message types
 *
 *****************************************************************************/

//types
#define gdGazeMsg				201	//send gaze message
#define gdControlMsg			202	//send control message
#define gdTextMsg				203	//send text

//subtypes gdGazeMsg
#define gdGazeSample			1	//send new eye tracking sample
#define gdGazeFixation			2	//send new eye tracking fixation
#define gdGazeBlink				3	//send new eye tracking blink

//subtypes gdControlMsg
#define gdPoke					1	//poke client
#define gdRegister				2	//register client
#define gdUnregister			3	//unregister client

//subtypes gdTextMsg
#define gdName					1	//name
#define gdDescription			2	//description
#define gdText					3	//text message


/*-----------------------------------------------------------------------------
 * gdHdrMessage - header message
 *
 * This message holds source Id and amount of data entries, that follows this header.
 * Type of the data that follows this header depend on the type of the message.
 */
typedef struct _gdHdrMsg {
    CARD8 type;			// gdGazeMsg | gdControlMsg | gdTextMsg
    CARD8 subtype;		//subtype
	CARD16 id;			//source Id
	CARD16 reserved2;
	CARD16 count;		//amount of following data entries
} gdHdrMsg;

#define sz_gdHdrMsg 8


/* server -> client */
//Gaze Data messages
/*#define gdServerGazeSample			201	//send new eye tracking sample
#define gdServerGazeFixation		202	//send new eye tracking fixation
#define gdServerGazeBlink			203	//send new eye tracking blink

//Client "control" messages
#define gdServerNickname			204	//send new nickname
#define gdServerDescription			205	//send new description
#define gdServerTxtMessage			206	//send text message to the client

#define gdServerReg					207	//send add client to the tracking list
#define gdServerUnReg				208	//send remove client from the tracking list
#define gdServerPoke				209	//send client poke

/* client -> server */
//Gaze Data messages
/*#define gdClientGazeSample			251	//send new eye tracking sample
#define gdClientGazeFixation		252	//send new eye tracking fixation
#define gdClientGazeBlink			253	//send new eye tracking blink

//Client "control" messages
#define gdClientNickname			254	//send new nickname
#define gdClientDescription			255	//send new description
#define gdClientTxtMessage			256	//send text message

#define gdClientReg					257	//send add client to the tracking list
#define gdClientUnReg				258	//send remove client from the tracking list
#define gdClientPoke				259	//send poke
*/

/*****************************************************************************
 *
 * Server -> client message definitions
 *
 *****************************************************************************/



/*-----------------------------------------------------------------------------
 * gdServerHdrMsg - Send header message that contains.
 *
 * This message holds source Id and amount of data entries, that follows this header.
 * Type of the data that follows this header depend on the type of the message.
 */
/*typedef struct _gdServerHdrMsg {
    CARD8 type;			// gdServerGazeSample | gdServerGazeFixation | gdServerGazeBlink | gdServerNickname | gdServerDescription | gdServerMessage | gdServerReg | gdServerUnReg | gdServerPoke
    CARD8 subtype;		//subtype
	CARD16 id;			//source Id
	CARD16 reserved2;
	CARD16 count;		//amount of following data entries
} gdServerHdrMsg;

#define sz_gdServerHdrMsg 8


/*-----------------------------------------------------------------------------
 * gdServerGDHdrMsg - Send gaze data entries.
 *
 * This message is followed by "count" structures of gaze data entries
 */
/*typedef struct _gdServerGDHdrMsg {
    CARD8 type;	// gdServerGazeSample | gdServerGazeFixation | gdServerGazeBlink
    CARD8 pad1;
	CARD16 id;
	CARD16 pad2;
	CARD16 count;
} gdServerGDHdrMsg;

#define sz_gdServerGDHdrMsg 8

/*-----------------------------------------------------------------------------
 * gdServerTextHdrMsg - Send text.
 *
 * This message is followed by "length" characters of a text message
 */
/*typedef struct _gdServerTextHdrMsg {
    CARD8 type;	// gdServerNickname | gdServerDescription | gdServerMessage
	CARD8 pad1;
	CARD16 id;
	CARD16 pad2;
	CARD16 length;
} gdServerTextHdrMsg;

#define sz_gdServerTextHdrMsg 8

/*-----------------------------------------------------------------------------
 * gdServerIdMsg - Send id.
 *
 * This message send only client ID, that should be processed in some way
 */
/*typedef struct _gdServerIdMsg {
    CARD8 type;	// gdServerReg | gdServerUnReg | gdServerPoke
	CARD8 pad;
	CARD16 id;
} gdServerIdMsg;

#define sz_gdServerIdMsg 4*/


/*****************************************************************************
 *
 * Client -> server message definitions
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 * gdClientGDHdrMsg - Send gaze data entries.
 *
 * This message is followed by "count" structures of gaze data entries
 */
/*typedef struct _gdClientGDHdrMsg {
    CARD8 type;	// gdClientGazeSample | gdClientGazeFixation | gdClientGazeBlink
    CARD8 pad;
	CARD16 count;
} gdClientGDHdrMsg;

#define sz_gdClientGDHdrMsg 4

/*-----------------------------------------------------------------------------
 * gdClientTextHdrMsg - Send text.
 *
 * This message is followed by "length" characters of a text message
 */
/*typedef struct _gdClientTextHdrMsg {
    CARD8 type;	// gdClientNickname | gdClientDescription | gdClientMessage
	CARD8 pad1;
	CARD16 pad2;
	CARD16 dstid;
	CARD16 length;
} gdClientTextHdrMsg;

#define sz_gdClientTextHdrMsg 8

/*-----------------------------------------------------------------------------
 * gdClientIdMsg - Send id. This message is rceived from the client so do not contain ID
 * itself.
 *
 * This message send only client ID, that should be processed in some way
 */
/*typedef struct _gdClientIdMsg {
    CARD8 type;	// gdClientReg | gdClientUnReg | gdClientPoke
} gdClientIdMsg;

#define sz_gdClientIdMsg 1*/

/*typedef struct _gdClientHdrMsg {
    CARD8 type;			// gdClientGazeSample | gdClientGazeFixation | gdClientGazeBlink | gdClientNickname | gdClientDescription | gdClientMessage | gdClientReg | gdClientUnReg | gdClientPoke
    CARD8 subtype;
	CARD16 id;			//destination Id
	CARD16 reserved2;
	CARD16 count;		//amount of following data entries
} gdClientHdrMsg;

#define sz_gdClientHdrMsg 8*/
