/*-----------------------------------------------------------------------------
 * This file defines custom data type, used to store and process gaze data
 */
#pragma once

#ifdef _WIN32
#include <ctime>
#endif

#include "gdproto.h"
#include <ostream>

using namespace std;

#define MAX_NICKNAME_LENGTH 8
#define MAX_DESCRIPTION_LENGTH 256

#define DEFAULT_LOG_FOLDER "C:\\GDLOG"

//Gaze Data Items' types
typedef enum _gdElementType
{
	GDT_UNDEFINED,		
	GDT_SAMPLE,			
	GDT_SINGLESAMPLE,
	GDT_FIXATION,
	GDT_BLINK,
	GDT_EVENT			
} gdElementType;

//Gaze Data Items' types
typedef enum _gdLogRecordType
{
	LRT_UNDEFINED	= 0x000,
	LRT_GAZEDATA	= 0x001,
	LRT_MESSAGE		= 0x002,
	LRT_REGCLIENT	= 0x003,
	LRT_UNREGCLIENT	= 0x004,
	LRT_POKECLIENT	= 0x005,
	LRT_STARTTRACK	= 0x006,
	LRT_SETNICKNAME	= 0x007,
	LRT_SETDESCRIPT	= 0x008
} gdLogRecordType;

typedef enum _gdEye
{
	GDE_UNDEFINED = -1,
	GDE_LEFT = 0,
	GDE_RIGHT = 1
} gdEye;


/****************************************************************************/
/*	Protocol messages gaze data types										*/
/****************************************************************************/

/**
 * Sample gaze data for one eye.
 */
typedef struct _gdSingleSample
{
	CARD32 number;		//ordinal number
	CARD32 time;		//time offset from the start of the record
	CARD16 eye;			//event code
	CARD16 pad;			//reserved
	CARD32 x;			//x coordinate of the eye
	CARD32 y;			//y coordinate of the eye
	CARD32 pupil;		//pupil size
} gdSingleSample;

#define sz_gdSingleSample 24

/**
 * Fixation gaze data.
 */
typedef struct _gdFixation
{
	CARD32 number;		//ordinal number
	CARD32 time;		//time offset from the start of the record
	CARD32 duration;	//fixation duration
	CARD16 eye;			//eye number
	CARD16 pad;
	CARD32 x;			//x coordinate of the eye
	CARD32 y;			//y coordinate of the eye
} gdFixation;

#define sz_gdFixation 24

/**
 * Blink gaze data.
 */
typedef struct _gdBlink
{
	CARD32 number;		//ordinal number
	CARD32 time;		//time offset from the start of the record
	CARD32 duration;	//fixation duration
	CARD16 eye;			//eye number
	CARD16 pad;
} gdBlink;

#define sz_gdBlink 16


struct TimeStamp
{
	unsigned short Year;	//current year
	unsigned int Days;	//days past since January 1st
	unsigned long Milliseconds;	//Milliseconds past since 00:00 for the current day

public:
	TimeStamp()
	{
#ifdef _WIN32
	time_t systime = time(NULL);
	tm* t = gmtime(&systime);
	
	Year = t->tm_year;
	Days = t->tm_yday;
	Milliseconds = (t->tm_hour*3600 + t->tm_min*60 + t->tm_sec) * 1000;
#endif
	}

	TimeStamp(unsigned short Y, unsigned int D, unsigned long M):
		Year(Y), Days(D), Milliseconds(M)
	{}

	friend ostream &operator<<(ostream &stream, TimeStamp ts)
	{
		stream << ts.Year << "-";
		stream << ts.Days << "-";
		stream << ts.Milliseconds;
  
		return stream;
	}
};
